/*
 This directive is special because it its tightly integrated in the Template binding system. 
 */
define(["js!js/libs/jQuery/QTransform.js"] , function(){

	var TYPES = ["fade","slide-h","slide-v"];

	var IDLE = "idle",
		ANIM_OUT = "animOut",
		ANIM_IN = "animIn",
		WAITING = "waiting";
	
	var Animation = function(layout , binding){
		this.state = IDLE;
		
		this.layout = layout;
		this.binding = binding;
		
		this.$element = binding.$element;
		
		this.type = this.$element.attr("bind-animation");
		this.duration = this.$element.attr("bind-animation-duration") || 400;
		this.condition = this.$element.attr("bind-animation-condition");

		this.locked = false;
		
		this.lastValue = null;
		
		if(!this.type){
			throw new Error();
		}
	};
	
	Animation.prototype.handleModelChanges = function(){
		if(AF.reader.sizeChanged){
			this.resetAnim();
			return false; //we don't want to animate after a rotation
		}else{
			this.locked = true;
			if(this.state == IDLE || this.state == ANIM_IN){
				this._animateOut(1);
			}
			return true;
		}
	};

	Animation.prototype.hide = function(){
		this.resetAnim();
	};

	Animation.prototype.resetAnim = function(){
		//kill animation
		this.$element.stop(true);
		this.state = IDLE;
		
		this.$element.css({
			"opacity": 1 ,
			"translateX": 0 ,
			"translateY": 0
		});
	};
	

	Animation.prototype.check = function(){
		if(this.locked && this.state == WAITING){
			this._checkCondition();
		}
	};
	
	

	Animation.prototype._animateOut = function(){
		this.state = ANIM_OUT;
		
		this.$element.stop(true);
		
		switch(this.type){
			
			case "slide-h":
				var newX= -this.$element.width() * this._getDirection();
				this.$element.animate( {translateX:newX } , this.duration , "swing" , $.proxy(this._animateOutFinished , this));
			break;
			
			case "slide-v":
				var newY= this.$element.height() * this._getDirection();
				this.$element.animate( {translateY:newY } , this.duration , "swing" , $.proxy(this._animateOutFinished , this));
			break;
			
			case "fade":
				this.$element.animate( { opacity:0 } , this.duration , "swing" , $.proxy(this._animateOutFinished , this));
			break;
			default:
				throw new Error();
		}
	};

	//Returns 1 for next, -1 for previous
	Animation.prototype._getDirection = function(){
		if(this.binding.sourceObjectName=="section"){
			return AF.reader.sectionDirection;
		}else if(this.binding.sourceObjectName=="article"){
			return AF.reader.articleDirection;
		}else{
			return 1;
		}
	};
	
	
	Animation.prototype._animateOutFinished = function(){
		this.state = WAITING;

		if(this.condition){
			//re-apply binding that we stopped in the first place
			if(!this._checkCondition()){
				this.layout.applyBinding(this.binding , false);
			}
		}else{
			//re-apply binding that we stopped in the first place
			this.layout.applyBinding(this.binding , true);

			this._ready();
		}
	};

	
	Animation.prototype._checkCondition = function(){
		if(this.condition){
			if(AF.reader.models[this.condition] != null){
				this.layout.applyBinding(this.binding , true);
				//bind out is okay
				if(this.state==WAITING){
					this._ready();
				}
				return true;
			}
		}
		return false;
	};

	Animation.prototype._ready = function(){
		if(this.state == WAITING){
			this.locked = false;
			
			this._animateIn(1);
		}
	};
	
	
	Animation.prototype._animateIn = function(direction){
		this.state = ANIM_IN;
		
		this.$element.stop(true);

		switch(this.type){

			case "slide-h":
				var oldX = this.$element.width() * this._getDirection();
				this.$element.css("translateX",oldX);
				this.$element.animate( {translateX:0 } , this.duration , "swing" , $.proxy(this._animateInFinished , this));
				break;

			case "slide-v":
				var oldY = -this.$element.height() * this._getDirection();
				this.$element.css("translateY",oldY);
				this.$element.animate( {translateY:0 } , this.duration , "swing" , $.proxy(this._animateInFinished , this));
				break;

			case "fade":
				this.$element.animate( { opacity:1 } , this.duration , "swing" , $.proxy(this._animateInFinished , this));
				break;
		}
	};

	Animation.prototype._animateInFinished = function(){
		this.state = IDLE;
	};
	
	
	return Animation;
});
	
