/**
 * Created with JetBrains WebStorm.
 * User: yann
 * Date: 4/16/13
 * Time: 11:30 AM
 * To change this template use File | Settings | File Templates.
 */
define(["js/app/Animation","js!js/libs/jQuery/jquery.touchSwipe.js"] , function(Animation){

	//!!! jQuery extend !!!
	$.fn.findIncludingSelf = function(selector) {
		return this.find(selector).add(this.filter(selector));
	};

	
	var BINDINGS = [
		"template",
		"text",
		"image"
	];

	var ACTIONS = [
		"nextArticle",
		"prevArticle",
		"nextSection",
		"prevSection"
	];
	
	var Layout = function(orientation){
		this.orientation = orientation;
		
		this.$element = null;
		this.bindingsMap = null;  
		this.templateBindings = null;
		
		this.directives = null;
		
		this.displayed = false;
		this.parentElement = null;
	};

	Layout.prototype._log = function(fName){
		return "\t\tLayout["+this.orientation+"]#"+fName+" -";
	};

	// -------------------------------------------------------------------------------
	// Init methods
	// -------------------------------------------------------------------------------

	Layout.prototype.parseHTML = function(html){
		var self = this;
		var $layout = $(html);
		this.$element = $layout;
		
		//default css
		$layout.addClass("af-template-layout");
		
		if($layout.length != 1){
			AF.parseError("incorrect template HTML");
		}
		
		//get all bindings
		this.bindingsMap = {};
		this.templateBindings = [];
		$.each( BINDINGS , function(index , bindType){
			var bindAttr = "bind-"+bindType;
			$layout.find("*["+bindAttr+"]").each(function(index){
				var $element = $(this),
					bindingSourceSplit = $element.attr(bindAttr).split(" "),
					bindingModelObjectName = bindingSourceSplit[0],
					bindingModelProp = bindingSourceSplit[1];
				
				var binding = {
					$element: $element ,
					sourceObjectName: bindingModelObjectName ,
					sourceProp: bindingModelProp ,
					sourceType: bindType
				};
				
				if(!self.bindingsMap[bindingModelObjectName]){
					self.bindingsMap[bindingModelObjectName] = [];
				}
				self.bindingsMap[bindingModelObjectName].push(binding);
				if(bindType=="template"){
					self.templateBindings.push(binding);
				}
				
				//Animation
				var animation = $element.attr("bind-animation");
				if(animation){
					binding.animation = new Animation(self,binding);
				}
				
			});
		});

		/*
		//plug actions
		$.each( ACTIONS , function(index , actionType){
			var actionAttr = "action-"+actionType,
				fName = actionType,
				closure = function(){
					AF.reader[fName]();
				};
			$layout.find("*["+actionAttr+"]").click(closure);
		});
		*/
		var doAction = function(val){
			AF.reader.evalAction(val);
		};
		
		//CLICKS
		$layout.findIncludingSelf("*[on-click]").click(function(){
			var attr = $(this).attr("on-click");
			doAction(attr);
		});

		//SWIPE
		$layout.findIncludingSelf("*[on-swipe-left] , *[on-swipe-right]").each( function(){
			var $element = $(this);
			var swipeObj = {};
			var leftAttr = $element.attr("on-swipe-left"); 
			if(leftAttr){
				swipeObj.swipeLeft = function(){
					doAction(leftAttr);
				};
			}
			var rightAttr = $element.attr("on-swipe-right");
			if(rightAttr){
				swipeObj.swipeRight = function(){
					doAction(rightAttr);
				};
			}
			$element.swipe( swipeObj );
		});
		


		this.directives = [];
		$layout.findIncludingSelf("*[directive]").each( function(){
			//create jquery object
			var $element = $(this);

			//push required modules
			var directiveNameArray = $element.attr("directive").split(" ");
			for(var i=0 ; i<directiveNameArray.length ; i++){
				self.directives.push({
					$element : $element ,
					name: directiveNameArray[i],
					directive: null
				});
			}
		});
	};


	Layout.prototype.initDirectives = function(html){
		
		for(var i=0 ; i<this.directives.length ; i++){
			var directiveData = this.directives[i];
			var directiveClass = AF.directives[directiveData.name];
			directiveData.directive = new directiveClass();
			directiveData.order = directiveClass.order || 0;
	
			//this allows us to access directive from DOM element
			directiveData.$element.data(directiveData.name , directiveData.directive);
		}
	
		//sort directives depending on their order of priority
		this.directives.sort(function(d1,d2){
			return d1.order - d2.order;
		});
	
		//init them in the defined order
		//two pass because we need two init methods in order to handle complex dependency cases
		for(var k=0 ; k<2 ; k++){
			for(i=0 ; i<this.directives.length ; i++){
				directiveData = this.directives[i];
				if(k==0){
					directiveData.directive.init(directiveData.$element);
				}else{
					if(directiveData.directive.init2){
						directiveData.directive.init2(directiveData.$element);
					}
				}
			}
		}
	};


	// -------------------------------------------------------------------------------
	// "Private" update methods
	// -------------------------------------------------------------------------------
	
	Layout.prototype.display = function(parentElement){
		//make sure we are in the correct parent
		if(parentElement != this.parentElement){
			if(this.parentElement){
				this.$element.detach();
			}
			this.parentElement = parentElement;
			this.$element.appendTo(parentElement);
		}

		//make sure css display is correct
		if(!this.displayed){
			this.$element.css("display","block");
			this.displayed = true;
			
			//if we weren't dispalyed already, we have to make sure model is up to date
			this.handleModelChanged();
		}
	};

	Layout.prototype.hide = function(){
		if(this.displayed){
			this.displayed = false;
			this.$element.css("display","none");

			//TO DO: call "hide" on directives too ?
			
			for(var i=0 , len = this.templateBindings.length ; i<len ; i++){
				var binding = this.templateBindings[i]; 
				binding.currentTemplate.hide();
				if(binding.animation){
					binding.animation.hide();
				}
			}
		}
	};


	Layout.prototype.applyModelChanges = function(changeMap){
		var hasChanges = false;
		for(var bindingKey in this.bindingsMap){
			//for each binding that changed
			if(this.bindingsMap.hasOwnProperty(bindingKey)){
				if(!changeMap || changeMap.hasOwnProperty(bindingKey)){
					var bindings = this.bindingsMap[bindingKey];
	
					//apply binding
					var iLen = bindings.length;
					for(var i=0 ; i<iLen ; i++){
						var binding = bindings[i];
						
						if(!binding.animation || !binding.animation.handleModelChanges()){
							this.applyBinding(binding);
						}
						hasChanges = true;
					}
				}
			}
		}
		
		//apply changes on directives
		iLen = this.directives.length;
		for(i=0 ; i<iLen ; i++){
			var directive = this.directives[i].directive;
			if(directive.applyModelChanges){
				hasChanges = directive.applyModelChanges(changeMap) ||  hasChanges;
			}
		}
		
		//Note: has changes is not used for the moment
		//The goal was to pinpoint elements that required an update, but that is too hard to do,
		//Any change may have repercussions on different levels
		return hasChanges;
	};

	Layout.prototype.applyBinding = function(binding , afterAnim){
		switch(binding.sourceType){
			
			case "text":
				this.applyTextBinding(binding);
			break;

			case "template":
				this.applyTemplateBinding(binding);
				if(afterAnim){
					var wasDisplayed = binding.currentTemplate.currentLayout && binding.currentTemplate.currentLayout.displayed; 
					binding.currentTemplate.display(binding.$element[0]);
					if(wasDisplayed){
						//fixes bug when changing model during an animOut
						//TO DO: see if there is a cleaner way of doing this
						binding.currentTemplate.handleModelChanged();
					}
				}
			break;
			
			case "image":
				this.applyImageBinding(binding);
			break;
		}
	};
	
	Layout.prototype.applyTextBinding = function(binding){
		var obj = AF.reader.models[binding.sourceObjectName];
		
		var value;
		if( obj && obj["texts"].hasOwnProperty(binding.sourceProp) ){
			value = obj["texts"][binding.sourceProp];
		}else{
			value = "";
		}

		binding.$element.html( value );
	};
	
	Layout.prototype.applyTemplateBinding = function(binding){
		var obj = AF.reader.models[binding.sourceObjectName];
		
		var currentTemplate = binding.currentTemplate;
		var requiredTemplate = AF.reader.getTemplateForModel(binding.sourceObjectName);
		
		if(currentTemplate && currentTemplate!=requiredTemplate){
			currentTemplate.hide();
		}

		binding.currentTemplate = requiredTemplate;
	};

	Layout.prototype.applyImageBinding = function(binding){
		var obj = AF.reader.models[binding.sourceObjectName];

		var value = null;
		if( obj && obj["media"].hasOwnProperty(binding.sourceProp) ){
			value = obj["media"][binding.sourceProp];
			value = AF.reader.getImageUrl(value);
		}

		var imageDirective = binding.$element.data("image");
		
		if(imageDirective){
			imageDirective.setSrc(value);
		}else{
			//assume img tag
			if(value){
				binding.$element.css("visibility" , "none");
			}else{
				binding.$element.css("visibility" , "visible");
				binding.$element[0].src = value;
			}
		}
	};
	
	
	//Called: after applyModelChanges & after window resize
	Layout.prototype.update = function(){
		//update all directives
		for(var i=0 , len=this.directives.length ; i<len ; i++){
			this.directives[i].directive.update();
		}
	};

	// -------------------------------------------------------------------------------
	// Public methods
	// -------------------------------------------------------------------------------
	
	
	//called when model changed and after display 
	Layout.prototype.handleModelChanged = function(changeMap){
		//update model
		var hasChanges = this.applyModelChanges(changeMap);
		
		//update layout
		if(hasChanges || !changeMap){
			this.update();
		}
		
		//propagate to children
		for(var i=0 , len = this.templateBindings.length ; i<len ; i++){
			var templateBinding = this.templateBindings[i];
			if(templateBinding.currentTemplate){
				//we can only propagate chnages on templates that are not locked by the animation
				if(!templateBinding.animation || !templateBinding.animation.locked){
					if(!templateBinding.currentTemplate.displayed){
						templateBinding.currentTemplate.display(templateBinding.$element[0]);
					}else{
						templateBinding.currentTemplate.handleModelChanged(changeMap);
					}
				}else{
					//if locked by animation, we want to check if bind out is ok
					if(templateBinding.animation){
						templateBinding.animation.check();
					}
				}
			}
		}
	};

	Layout.prototype.handleSizeChanged = function(changeMap){
		this.update();
		
		//propagate to children
		for(var i=0 , len = this.templateBindings.length ; i<len ; i++){
			this.templateBindings[i].currentTemplate.handleSizeChanged();
		}
	};
	
	

	return Layout;
});
