//singleton
define(["js/app/Layout"] , function(Layout){
	
	var Template = function(name , data){
		this.name = name;

		//Init CSS 
		this.css = data.css;
		for(var orientation in this.css){
			if(this.css.hasOwnProperty(orientation)){
				this.css[orientation].disabled = true;
			}
		}
		
		//Init HTML - build layouts from data.html
		this.currentLayout = null; //currently displayed template element
		this.layouts = {}; //compiled templates
		for(orientation in data.html){
			if(data.html.hasOwnProperty(orientation)){
				var layout = new Layout( orientation );
				layout.parseHTML(data.html[orientation]);
				this.layouts[orientation] = layout;
			}
		}
		
		this.currentLayout = null;
		this.currentCSS = null;
		
		this.currentParent = null;
		this.displayed = false;
	};
	
	
	//used at startup in order to know what needs to be preloaded
	Template.prototype.getRequiredDirectives = function(){
		var requiredDirectivesMap = {};
		for(var orientation in this.layouts){
			if(this.layouts.hasOwnProperty(orientation)){
				var layout = this.layouts[orientation];
				for(var i=0 , len=layout.directives.length ; i<len ; i++){
					requiredDirectivesMap[ layout.directives[i].name ] = true;
				}	
			}
		}
		var directives = [];
		for(var directiveName in requiredDirectivesMap){
			if(requiredDirectivesMap.hasOwnProperty(directiveName)){
				directives.push( directiveName );
			}
		}
		return directives;
	};
	
	Template.prototype._log = function(fName){
		return "\tTemplate["+this.name+"]#"+fName+" -";
	};
	
	// -------------------------------------------------------------------------------
	// INIT
	// -------------------------------------------------------------------------------
	
	//called when all required modules are finished loading
	//finishes initialization
	Template.prototype.init = function(){
		for(var orientation in this.layouts){
			if(this.layouts.hasOwnProperty(orientation)){
				var layout = this.layouts[orientation];
				layout.initDirectives();
			}
		}
	};
	
	// -------------------------------------------------------------------------------
	// CHANGING DISPLAY
	// -------------------------------------------------------------------------------
	
	//Changes parent or orientation
	Template.prototype.display = function(parentElement){
		var changed = false;

		//Make parameter optional
		if(!parentElement){
			parentElement = this.currentParent; 
		}else{
			this.currentParent = parentElement;
		}
		if(!parentElement){ 
			throw new Error("no parent :'(");
		}
		
		if(this.css){
			//switch CSS
			var newCSS;
			if(this.css.both){
				newCSS = this.css.both;
			}else{
				newCSS = AF.reader.isHorizontal ? this.css.horizontal : this.css.vertical;
			}
			if(this.currentCSS && newCSS!=this.currentCSS){
				this.currentCSS.disabled = true;
			}
			this.currentCSS = newCSS;
			this.currentCSS.disabled = false;
		}	
		
		//switch layout
		var newLayout;
		if(this.layouts.both){
			newLayout = this.layouts.both;
		}else{
			newLayout = AF.reader.isHorizontal ? this.layouts.horizontal : this.layouts.vertical; 
		}
		changed = newLayout!=this.currentLayout;
		if(this.currentLayout && newLayout!=this.currentLayout){
			this.currentLayout.hide();
		}
		this.currentLayout = newLayout;
		this.currentLayout.display(parentElement);
		
		this.displayed = true;
		
		return changed;
	};
	
	Template.prototype.hide = function(){
		if(this.currentCSS){
			this.currentCSS.disabled = true;
		}
		if(this.currentLayout){
			this.currentLayout.hide();	
		}

		this.displayed = false;
	};
	
	// -------------------------------------------------------------------------------
	// Public API
	// -------------------------------------------------------------------------------
	
	/*
	Template.prototype.init = function(parentElement){
		//make sure we are displayed in the correct parent
		this.display(parentElement);
	};
	*/

	//Call this after a possible DOM or size change
	Template.prototype.handleSizeChanged = function(){
		var changedLayout = this.display();
		if(!changedLayout){
			this.currentLayout.handleSizeChanged();
		}
	};
	
	//change map is optional, if note defined, updates everything
	Template.prototype.handleModelChanged = function(changeMap){
		this.currentLayout.handleModelChanged(changeMap);
	};
	

	return Template;
});
