define(["js!js/libs/FTColumnflow"], function(){
	
	
	var ColumnsFT = function(){
		this.$element = null;
		this.$content = null;
		this.$flow = null;
		this.$fixed = null;
		
		this._reflowObj = null;
		
		this._shouldRecreate = false;
		
		this._selectedPillIndex = -1;
	};

	ColumnsFT.prototype.init = function($element){
		this.$element = $element;
		
		this.$content = $element.find(".columnsFT-content");
		this.$flow = $element.find(".columnsFT-flow");
		this.$fixed = $element.find(".columnsFT-fixed");
		
		if( this.$content.length !=1
			|| this.$flow.length !=1 ){
			//|| this.$fixed.length !=1){
			AF.parseError("missing reflow children");
		}
		
		this.$flow.css("display","none");
		this.$fixed.css("display","none");

		//Optional for pills
		this.$pillHolder = $element.find(".columnsFT-pills");
		if(this.$pillHolder.length==1){
			var $pillElement = this.$pillHolder.children();
			if($pillElement.length == 1){
				this.selectedPillIndex = 0;
				this.pillHtml = $pillElement[0].outerHTML;
				$pillElement.remove();
			}else{
				this.$pillHolder = null;
			}
		}else{
			this.$pillHolder = null;
		}
	};

	ColumnsFT.prototype.init2 = function(){
		if(this.$pillHolder){
			this.scroller =  this.$element.find(".scrollWrapper").data("af-scroll");
			if(this.scroller){
				this.scroller.scrollSignal.add(this._onScroll , this);
			}
		}
	};
	
	ColumnsFT.prototype.applyModelChanges = function(changeMap){
		this._shouldRecreate = true;
		
		//not needed content is supposed to use bindings
		return false;
	};

	ColumnsFT.prototype.update = function(){
		this._reflow(this._shouldRecreate);
		this._shouldRecreate = false;
	};


	ColumnsFT.prototype._reflow = function(recreate){
		var config = {
			columnWidth: this._getColumnWidth(),
			pageArrangement: 'horizontal',
			standardiseLineHeight: true,
			columnGap: 20,
			pagePadding: 10,
			minFixedPadding: 0/*,
			debug: true,
			showGrid: true*/
		};
	
		//$reflowContent.css("font-size",reader.fontSize);
	
		try{
			if(recreate || !this._reflowObj){
				if(this._reflowObj){
					//this._reflowObj.destroy(); //seems bugged
					$('style').last().remove(); //should do the same, roughly
				}
				
				//Create
				this._reflowObj = new FTColumnflow( this.$content[0], this.$element[0] , config);
				this._reflowObj.flow( this.$flow[0] , this.$fixed[0] );
			
				//$contentHolderElement.css("display","block");
				
				if(this.scroller){
					this.scroller.setPosition(0,0)
				}
				
				
			}else{
				//trying to fix: + zoom in + 6 zoom out -> data-cf-original-padding is changed and layout is messed up
				this.$element.find(".cf-column > p").css("padding",0);
				
				//Update
				this._reflowObj.reflow(config)
				
			}
		}catch(e){
			//console.warn(e);
		}
		
		var self = this;
		setTimeout(function(){
			//We need to leave the reflow some breathing time
			self._updatePills();
		},0);
	};
	
	ColumnsFT.prototype._getColumnWidth = function(){
		
		var columnWidth = 250; //for a font size of 12
		//columnWidth *= reader.fontSize/12; //make column bigger proportionnally to font increase
		//this is the ideal column size, it just can't be bigger than screen width
		var width = this.$element.width();
		if(columnWidth > width){
			columnWidth = width;
		}
		return columnWidth;
		
	};

	
	ColumnsFT.prototype._updatePills = function(){
		if(this.$pillHolder && this._reflowObj){
			this.$pillHolder.css("display","none");
			
			var numWantedPills = this._reflowObj.pagedContentCount,
				$currentPills = this.$pillHolder.children(),
				len = $currentPills.length,
				i;

			if(numWantedPills>len){
				var pillsArray = [];
				//Create missing pills
				for(i = len ; i< numWantedPills ; i++){
					var $pill = $(this.pillHtml);
					$pill.find(".columnsFT-pill-index").text( (i+1).toString() );
					$pill.click( this._getPillClickFunction(i) );
					$pill.appendTo(this.$pillHolder);
				}
			}

			//make sure all pills are displayed
			var pillsToShow = Math.min(numWantedPills,len);
			for(i = 0 ; i< pillsToShow ; i++){
				$currentPills[i].style.display = "";
			}
			for(i = pillsToShow ; i< len ; i++){
				$currentPills[i].style.display = "none";
			}

			this._selectedPillIndex = -1;
			this._updateSelectedPill();

			this.$pillHolder.css("display","block");
		}
	};

	ColumnsFT.prototype._updateSelectedPill = function(){
		if(this.$pillHolder && this.scroller){
			var index = Math.round( this.scroller.getPosition() / this.$element.width());
			if(index != this._selectedPillIndex){
				this._selectedPillIndex = index;
				
				var $currentPills = this.$pillHolder.children();
				$currentPills.removeClass("selected");
				$currentPills.eq(index).addClass("selected");
			}
		}
	};

	ColumnsFT.prototype._getPillClickFunction = function(i){
		var self = this;
		return function(){
			var position = i*self.$element.width();
			self.scroller.setPosition(position , 200);
		}
	};

	ColumnsFT.prototype._onScroll = function(){
		this._updateSelectedPill();
	};
	
	

	ColumnsFT.order = 1; //after scroll
	AF.directives.columnsFT = ColumnsFT;
});
