define(["signals","js!js/libs/ftscroller"] ,  function(signals){
	
	var HORIZONTAL = "horizontal",
		VERTICAL = "vertical";

	
	var ScrollFT = function(){
		this.scrollSignal = new signals.Signal();
		this.swipePlugin = null;
	};

	ScrollFT.prototype.init = function($element){
		this.direction = $element.attr("scroll-direction") || VERTICAL;
		this.$element = $element;
		
		//this.$element.css("overflow" , "hidden");
		
		this.$content = this.$element.children().first();
		/*
		if( this.$content.length != 1){
			AF.parseError("scroll: should have one child div only");
			//this.$content.css("overflow" , "hidden");
		}
		*/
		
		$element.data("af-scroll",this);
		
		var paginated = this.$element.attr("scrollFT-paginated");
		paginated = typeof paginated !== 'undefined' && paginated !== false
		
		this._ftScroller = new FTScroller( this.$element[0] , {
			scrollbars: true,
			scrollingX: this.direction==HORIZONTAL,
			scrollingY: this.direction==VERTICAL,
			alwaysScroll: false,
			paginatedSnap: paginated,
			snapping: paginated,
			bouncing: false
		});
		this._ftScroller.addEventListener("scroll", $.proxy(this._onFTScroll,this) );
		this._ftScroller.addEventListener("scrollstart", $.proxy(this._onFTScrollStart , this));
		this._ftScroller.addEventListener("scrollend", $.proxy(this._onFTScrollEnd , this));
		
		//Sets ftscroller_container size (fixes a bug when scrolling columned content
		if(this.direction==HORIZONTAL){
			this.$element.children().eq(0).css("height","100%");
		}else{
			this.$element.children().eq(0).css("width","100%");
		}
	};
	

	ScrollFT.prototype.applyModelChanges = function(changeMap){
		return false;
	};
	
	ScrollFT.prototype.update = function(){
		if(this.direction == HORIZONTAL){
			var width = this.$content.innerWidth();
			this._ftScroller.updateDimensions( width , this.$element.height() );
		}else{
			var height = this.$content.innerHeight();
			this._ftScroller.updateDimensions( this.$element.width() , height );
		}
	};

	ScrollFT.prototype._onFTScroll = function(data){
		this.scrollSignal.dispatch();
	};

	ScrollFT.prototype._onFTScrollStart = function(){
		//look for a parent with a swipe plugin enabled
		var $node = this.$element,
			swipePlugin;
		while(!$node.is("body") && !swipePlugin){
			swipePlugin = $node.data("TouchSwipe");
			$node = $node.parent();
		}
		if(swipePlugin){
			this.swipePlugin = swipePlugin;
			this._ftScroller.addEventListener("scrollStart", $.proxy(this._onFTScrollStart,this) );
			this._ftScroller.addEventListener("scrollEnd", $.proxy(this._onFTScrollEnd,this) );
		}
		if(this.swipePlugin){
			this.swipePlugin.disable();
		}
	};

	ScrollFT.prototype._onFTScrollEnd = function(data){
		if(this.swipePlugin){
			this.swipePlugin.enable();
		}
	};
	

	ScrollFT.prototype.getPosition = function(){
		if(this.direction == HORIZONTAL){
			return this._ftScroller.scrollLeft || 0;
		}else{
			return this._ftScroller.scrollTop || 0;
		}	
	};
	
	ScrollFT.prototype.setPosition = function(value , animDuration){
		if(this.direction == HORIZONTAL){
			this._ftScroller.scrollTo(value , 0 , animDuration );
		}else{
			this._ftScroller.scrollTo( 0 , value , animDuration );
		}
	};

	ScrollFT.prototype.getElementPosition = function($element){
		var position = $element.position();
		if(this.direction == HORIZONTAL){
			return position.left;
		}else{
			return position.top;
		}
	};

	ScrollFT.order = 1; //needs to update AFTER all directives that may change content size (vbox, toc, ...)
	AF.directives.scrollFT = ScrollFT;
});
