define( function(){

	var HORIZONTAL = "horizontal",
		VERTICAL = "vertical";

	var Toc = function(){
		this.$element = null;
		this.orientation = null;

		this.$tocSectionArray = null;
		this.$tocArticleMap = null;

		this.$header = null;
		this.$headerSectionTitle = null;

		this.$selectedArticle = null;

		this.headerSection = null;

		this.scroller = null;
	};

	Toc.prototype.init = function($element){

		//Parse DOM

		this.$element = $element;

		this.orientation = $element.attr("toc-orientation");
		if(!this.orientation){
			this.orientation = VERTICAL;
		}
		this.$element.addClass("toc-"+this.orientation);
		
		this.onArticleClick = $element.attr("on-article-click");
		
		//content
		this.$contentHolder =  $element.find(".toc-contentHolder");
		this.$content = this.$contentHolder.find(".toc-content");

		var articleHTML , sectionHTML;
		var $section = this.$content.find(".toc-section");
		if($section.length>0){
			sectionHTML = $section[0].outerHTML;
			$section.remove();
		}
		var $article = this.$content.find(".toc-article");
		if($article.length>0){
			articleHTML = $article[0].outerHTML;
			$article.remove();
		}

		if(this.$content.length!=1 || this.$contentHolder.length!=1 || !sectionHTML || !articleHTML ){
			AF.parseError("toc directive");
		}

		//header (optional)
		//header is used to dispaly the section we are scrolling
		//it implies and requires that the content element as a scroller attached to it
		this.$header = $element.find(".toc-header");
		if(this.$header.length==1){
			this.$headerSectionTitle = this.$header.find(".toc-section-title");
			if(this.$headerSectionTitle.length == 0){
				this.$headerSectionTitle = null;
			}
		}else{
			this.$header = null;
		}

		//Build TOC

		//Start by cleaning up what was in toc
		this.$content.empty();

		//Templates are parsed AFTER issue model, so we know it is loaded
		var _issueModel = AF.reader.issueModel;

		this.$tocSectionArray = [];
		this.$tocArticleMap = {};

		for(var iSection = 0 , lenSection = _issueModel.sections.length ; iSection<lenSection ; iSection++){
			var section = _issueModel.sections[iSection];

			//Create section item
			var $section = $(sectionHTML);
			var $sectionTitle = $section.find(".toc-section-title");
			if($sectionTitle.length>0){
				$sectionTitle.html( section.texts.title);
			}
			this.$content.append($section);

			this.$tocSectionArray.push($section);

			for(var iArticle = 0 , lenArticle = section.articles.length ; iArticle<lenArticle ; iArticle++){
				var article = section.articles[iArticle];

				//Create article item
				var $article = $(articleHTML);
				var $articleTitle = $article.find(".toc-article-title");
				if($articleTitle.length>0){
					$articleTitle.html( article.texts.title);
				}
				this.$content.append($article);

				this.$tocArticleMap[article.id] = $article;

				$article.click( _getTocArticleClick(iArticle , iSection , this.onArticleClick) );
			}
		}
	};

	Toc.prototype.init2 = function($element){
		//wait for second init, when required scroller is initialized
		
		this.scroller = this.$contentHolder.data("af-scroll");
		if(!this.scroller){
			AF.parseError("toc directive: section title on header requires a scroll directive on contentHolder");
		}else{
			this.scroller.scrollSignal.add( this._refreshHeader , this);
		}
		this._refreshHeader();
	};

	var _getTocArticleClick = function( iArticle , iSection , onArticleClick){
		return function(){
			AF.reader.selectArticle(iArticle , iSection);
			if(onArticleClick){
				AF.reader.evalAction(onArticleClick);
			}
		}
	};

	Toc.prototype._refreshHeader = function(){
		if(this.$headerSectionTitle){

			var lenToc = this.$tocSectionArray.length,
				scrollSection = null,
				scrollPosition = this.scroller.getPosition();

			for(var iToc=0 ; iToc<lenToc ; iToc++){
				var $tocSection = this.$tocSectionArray[iToc],
					tocSectionPos = this.scroller.getElementPosition($tocSection);

				//Note that the test is differnet depending on the lib used (side effect of different div nesting)
				if( tocSectionPos >= scrollPosition ){
					//Section passes the ul top -> we need the one before
					if(iToc>0) iToc--;
					scrollSection = AF.reader.issueModel.sections[iToc]
					break;
				}
			}
			if(!scrollSection){
				scrollSection = AF.reader.issueModel.sections[lenToc-1];
			}

			if(scrollSection != this.scrollSection){
				this.scrollSection = scrollSection;

				this.$headerSectionTitle.html(this.scrollSection.texts.title);
				
				this.update(true);
			}
		}
	};

	Toc.prototype._keepSelectedArticleVisible = function(){
		if(this.scroller && this.$selectedArticle){
			var elementPos = this.scroller.getElementPosition(this.$selectedArticle) ,
				size = (this.scroller.direction == VERTICAL) ? this.$contentHolder.height() : this.$contentHolder.width() ,
				scrollPos = this.scroller.getPosition();
			
			var relPos = elementPos-scrollPos;
			if(relPos<10 || relPos>size - 10 - this.$selectedArticle.outerHeight()){
				this.scroller.setPosition(elementPos-20 , 100);	
			}
			
		}
	};


	Toc.prototype.applyModelChanges = function(changeMap){
		
		
		if(!changeMap || changeMap.article){
			if(this.$selectedArticle){
				this.$selectedArticle.removeClass("selected");
			}
			this.$selectedArticle = this.$tocArticleMap[AF.reader.models.article.id];
			if(this.$selectedArticle){
				this.$selectedArticle.addClass("selected");
			}
		}
		return false;
	};

	Toc.prototype.update = function(skipAutoScroll){
		if(this.$header){
			var h = this.$element.height() - this.$header.outerHeight();
			this.$contentHolder.height(h);
		}
		if(!skipAutoScroll){
			this._keepSelectedArticleVisible();
		}
	};

	Toc.order = 0; //needs to update before scroller  
	AF.directives.toc = Toc;
});
	
