//singleton
define( ["signals","js/app/Template"] , function(signals, Template){
	
	var DEVICE_DEFAULT = "default",
		DEVICE_PHONE = "phone";
	
	var reader = {
		fontSize : 16,
		issueModel : null,
		templateData : null,
		issueTemplate : null,
		isHorizontal : true,
		sectionDirection: 0,
		articleDirection: 0,
		sizeChanged: false, //Set to true during resize update, used by animation
		models : { // models bindings are based on
			article: null,
			articleContent: null,
			issue: null,
			section: null
		},
		state: { // used internally to manage current state
			sectionIndex: -1,
			articleIndex: -1,
			section: null,
			article: null
		}
	};
	
	var _issueModel, // Shortcut to reader.models.issue
		_debugDelay = 0 ,
		_state = reader.state;
	
	// -------------------------------------------------------------------------------
	// INITIALIZATION
	// -------------------------------------------------------------------------------
	
	reader.loadIssue = function(issueFile,templateFolder,articleId){
		var __t = Date.now();
		
		var slashIndex = issueFile.lastIndexOf("/");
		if(slashIndex<0){
			reader.issueFolder = "/";
		}else{
			reader.issueFolder = issueFile.slice(0,slashIndex+1); //take the slash
		}
		
		reader.templateFolder = templateFolder+"/";

		curl( ['text!'+issueFile] , function(data){
			__t = Date.now();
			
			//save model (dispatch changes)
			try{
				reader.issueModel = _issueModel = JSON.parse(data);
			}catch(e){
			}
			reader.setModel( "issue", _issueModel );
			
			//load issue template
			_loadTemplate(reader.templateFolder , function(template){
				_updateFontSize();
				
				if(articleId){
					reader.gotoArticle(articleId);
				}else{
					reader.selectArticle(0,0);
				}
				
				reader.issueTemplate = template; //used as a "isStarted" flag
				template.display( document.body );

				AF.reader.notify(AF.ReaderNotifications.LOADED);
			});
		});
	};
	
	// -------------------------------------------------------------------------------
	// Template loading
	// -------------------------------------------------------------------------------
	
	
	var _getJsonFromDeviceMap = function(deviceMap){
		//Debug
		//return deviceMap[DEVICE_PHONE];
		
		var deviceType = DEVICE_DEFAULT,
			screenWidth = window.screen.availWidth,
			screenHeight = window.screen.availHeight;
		if(!isNaN(screenWidth) && !isNaN(screenHeight)){
			if(screenWidth<640 || screenHeight<640){
				deviceType = DEVICE_PHONE;
			}
		}

		if(deviceMap[deviceType]){
			//return the data corresponding to the device 
			return deviceMap[deviceType];
		}else if(deviceMap[DEVICE_DEFAULT]){
			//return the default data
			return deviceMap[DEVICE_DEFAULT];
		}else{
			//at this point, just return the first data in the map, there probably is a problem in the template anyway
			for(var key in deviceMap){
				return deviceMap[key];
			}
		}
		return null;
	};
	
	var _loadTemplate = function(templateFolder , callback){
		
		curl([ "text!"+templateFolder+"template.json" ])
			.then(
			function (templateText) {
				var json = JSON.parse(templateText);
				
				//device map is optional
				if(json.deviceMap){
					var deviceJson = _getJsonFromDeviceMap(json.deviceMap);
					if(deviceJson){
						json = deviceJson;
					}
				}
				
				//font size is optional
				if(json.baseFontSize){
					reader.fontSize = parseInt(json.baseFontSize); 
				}
				
				var pathsToLoad = [],
					dataNodes = [ {target:"css",prefix:"css!"} , {target:"html",prefix:"text!"}];
				
				for(var templateName in json.templates){
					if(json.templates.hasOwnProperty(templateName)){
						var templateJson = json.templates[templateName];
		
						//Validate the css and html json and get the paths that we need to load
						$.each(dataNodes , function(index , value){
							if(typeof templateJson[value.target] == "string"){
								templateJson[value.target] = { both: templateJson[value.target]};
							}
							var targetObj = templateJson[value.target];
							
							/*
							if(typeof targetObj !== "object" ||
								(!targetObj.both && !targetObj.horizontal && !targetObj.vertical ) ){
								//incorrect
								AF.parseError("incorrect template content");
							}
							*/
							
							if(targetObj){
								for(var key in targetObj){
									if(targetObj.hasOwnProperty(key)){
										pathsToLoad.push( value.prefix+templateFolder+templateName+"/"+targetObj[key] );
										targetObj[key] = pathsToLoad.length-1; //We save an index that will be used to math the path with the curl args
									}
								}
							}
						});
						if(!templateJson.html){
							AF.parseError("incorrect template content");
						}
					}
				}

				if(json.commonCSS){
					pathsToLoad.push("css!"+templateFolder+json.commonCSS);
				}
	
				curl( pathsToLoad , function(){
					var requiredDirectives = [];
					//match paths with arguments by using the indexes
					for(var templateName in json.templates){
						if(json.templates.hasOwnProperty(templateName)){
							var templateJson = json.templates[templateName];
							for(var key in templateJson){
								if(templateJson.hasOwnProperty(key) && (key=="css" || key=="html")){
									for(var key2 in templateJson[key]){
										if(templateJson[key].hasOwnProperty(key2) && (key2=="both" || key2=="vertical" || key2=="horizontal")){
											templateJson[key][key2] = arguments[templateJson[key][key2]];
										}
									}
								}
							}
							var template = new Template(templateName , templateJson);
							json.templates[templateName] = template;
							requiredDirectives = requiredDirectives.concat(template.getRequiredDirectives());
						}
					}
					
					//json
					json.issueTemplate = json.templates[json.issueTemplate];
					json.defaultSectionTemplate = json.templates[json.defaultSectionTemplate];
					json.defaultSectionTemplate2 = json.templates[json.defaultSectionTemplate2];
					json.defaultArticleTemplate = json.templates[json.defaultArticleTemplate];
					json.defaultArticleTemplate2 = json.templates[json.defaultArticleTemplate2];
					
					reader.templateData = json;
					
					
					var filteredDirectives = [];
					for(var i=0 ; i<requiredDirectives.length ; i++){
						var path = "js/app/directives/"+requiredDirectives[i]+".directive";
						if(filteredDirectives.indexOf(path)){
							filteredDirectives.push( path );
						}
					}
					
					var finishedFunction = function(){
						for(var templateName in json.templates){ 
							var template = json.templates[templateName];
							template.init();
						}
						callback(reader.templateData.issueTemplate);
					};
					curl( filteredDirectives , finishedFunction);
				});
	
			},
			// execute this callback if there was a problem
			AF.curlError
		);
	};

	reader.getTemplateForModel = function(sourceName){
		var modelObj = reader.models[sourceName];
		if(modelObj){
			var templateName = modelObj.template;
			if(!templateName){
				if(sourceName=="section"){
					if(!reader.templateData.defaultSectionTemplate2 || _state.sectionIndex%2==0){
						return reader.templateData.defaultSectionTemplate;
					}else{
						return reader.templateData.defaultSectionTemplate2;
					}
				}else if(sourceName=="article"){
					if(!reader.templateData.defaultArticleTemplate2 || _state.articleIndex%2==0){
						return reader.templateData.defaultArticleTemplate;
					}else{
						return reader.templateData.defaultArticleTemplate2;
					}
				}
			}else{
				return reader.templateData.templates[templateName];
			}
			return templateName;
		}
		return null;
	};

	reader.getImageUrl = function(fileName){
		if(fileName && fileName!=""){
			return AF.reader.issueFolder+fileName;
		}else{
			return null;
		}
	};
	
	
	// -------------------------------------------------------------------------------
	// Article loading
	// -------------------------------------------------------------------------------
	
	
	//When the first article content has been loaded we force a resize in ardor to avoid IE glitches
	//TODO: see a cleaner way to solve this or at least use it only on IE
	var __IE9HackExecutedOnce = false;
	
	var _loadCurrentArticle = function(){

		var loadingArticle = _state.article;
		var _handleArticleLoaded =  function(data){
			if(loadingArticle == _state.article){ //check this still is the article we need
				var loadedArticle = JSON.parse(data);
				reader.setModel("articleContent",loadedArticle);
				
				if(!__IE9HackExecutedOnce){
					__IE9HackExecutedOnce = true;
					__IE9HackExecutedOnce = true;
					setTimeout( function(){
						_handleReaderResize();
					} , 1000);
				}
			}
		};

		var _callback;
		if(_debugDelay > 0){
			_callback = function(data){
				setTimeout( function(){
					_handleArticleLoaded(data);
				}, _debugDelay);
			}
		}else{
			_callback = _handleArticleLoaded;
		}

		curl( ['text!'+reader.issueFolder+"/articles/"+_state.article.id] , _callback);
        reader.notify(AF.ReaderNotifications.CURRENT, _state.article.id);
	};
	
	// -------------------------------------------------------------------------------
	// WINDOW RESIZE
	// -------------------------------------------------------------------------------

	var _resizeTimeoutId;
	var _handleWindowResize = function(){
		var $body = jQuery("body");
		reader.isHorizontal = $body.width() > $body.height();
		if(reader.issueTemplate){
			_handleReaderResize();
		}
	};
	$(window).resize(function(){
		clearTimeout(_resizeTimeoutId);
		_resizeTimeoutId = setTimeout(_handleWindowResize , 200 );
	});
	_handleWindowResize();
	
	var _handleReaderResize = function(){
		var __t = Date.now();
		reader.sizeChanged = true;
		reader.issueTemplate.handleSizeChanged();
		reader.sizeChanged = false;
	};

	reader._debug_switchOrientation = function(){
		reader.isHorizontal = !reader.isHorizontal;
		_handleReaderResize()
	};
	
	// -------------------------------------------------------------------------------
	// MODEL VALIDATION
	// -------------------------------------------------------------------------------

	reader.setModel = function(propOrMap , value){
		var key , map;
		//We want to work on a map
		if(typeof propOrMap=="string"){
			map = {};
			map[propOrMap] = value;
		}else{
			map = propOrMap;
		}
		
		//update changes
		var changes = false;
		for(key in map){
			if(map.hasOwnProperty(key) && reader.models[key] != map[key]){
				reader.models[key] = map[key];
				changes = true;
			} 
		}
		
		if(changes && reader.issueTemplate){
			var __t = Date.now();
			reader.issueTemplate.handleModelChanged(map);
		}
	};
	
	

	// -----------------------------------------------------------------------------------------------------------------
	// READER CONTROL
	// -----------------------------------------------------------------------------------------------------------------
	
	//used for on-* action bindings
	reader.evalAction = function(val){
		if(val.indexOf("(")<0){
			//call function by its name
			AF.reader[val]();
		}else{
			//call function with parameters
			try{
				eval("AF.reader."+val); //(lazy approach)
			}catch(e){
				//note this does not actually throw an error
				//console.error(e);
			}
		}
	};
	
	reader.showHide = function(selector){
		var $element = $(selector);
		if($element.hasClass("hide")){
			$element.removeClass("hide");
			$element.addClass("show");
		}else{
			$element.addClass("hide");
			$element.removeClass("show");
		}
	};
	
	reader.getSectionIndex = function(){
		return _state.sectionIndex;
	};

	reader.getArticleIndex = function(){
		return _state.articleIndex;
	};

	reader.getCurrentArticleId = function(){
		if(_state.article){
			return _state.article.id;
		}else{
			return null;
		}
	};

	reader.nextSection = function(){
		if(_state.sectionIndex < _issueModel.sections.length-1){
			reader.selectArticle(0,_state.sectionIndex+1);
		}
	};

	reader.prevSection = function(){
		if(_state.sectionIndex > 0){
			reader.selectArticle(0,_state.sectionIndex-1);
		}
	};

	reader.nextArticle = function(){
		if(_state.section){
			if(_state.articleIndex < _state.section.articles.length-1){
				reader.selectArticle(_state.articleIndex+1);
			}else{
				reader.nextSection();
			}
		}
	};

	reader.prevArticle = function(){
		if(_state.section){
			if(_state.articleIndex>0){
				reader.selectArticle(_state.articleIndex-1);
			}else if(_state.sectionIndex>0){
				reader.selectArticle(_issueModel.sections[_state.sectionIndex-1].articles.length-1 , _state.sectionIndex-1);
			}
		}
	};


	reader.gotoArticle = function(articleId){
		for(var i=0 , iLen=_issueModel.sections.length ; i<iLen ; i++){
			var articles = _issueModel.sections[i].articles;
			for(var j=0 , jLen=articles.length ; j<jLen ; j++){
				if(articles[j].id == articleId){
					reader.selectArticle( j , i);
					return;
				}
			}
		}
	};

	reader.selectArticle = function(articleIndex , sectionIndex){
		if(!_issueModel) return;

		var prevSectionIndex = _state.sectionIndex,
			prevArticleIndex = _state.articleIndex;

		//Change currect section if needed
		var sectionChanged = false;
		if(arguments.length>1){
			sectionIndex = parseInt(sectionIndex);

			if(!isNaN(sectionIndex)
				&& sectionIndex != _state.sectionIndex
				&& sectionIndex >=0
				&& sectionIndex <_issueModel.sections.length){

				//for animation, we need to know in which direction the change goes
				reader.sectionDirection = _state.sectionIndex<sectionIndex ? 1 : -1;
				
				//change current section
				_state.sectionIndex = sectionIndex;
				_state.section = _issueModel.sections[sectionIndex];
				sectionChanged = true;
			}
		}

		if(_state.section){

			//if we have a section selected, look, at the article index
			articleIndex = parseInt(articleIndex);

			var articleChanged = false;
			if(!isNaN(articleIndex)
				&& articleIndex!=_state.articleIndex
				&& articleIndex>=0
				&& articleIndex<_state.section.articles.length){
				//article index is a correct value
				articleChanged = true;
			}else if(sectionChanged){
				//we need a correct value because we changed section
				articleIndex = 0;
				articleChanged = true;
			}

			if(articleChanged){
				if(sectionChanged){
					reader.articleDirection = reader.sectionDirection;
				}else{
					reader.articleDirection = _state.articleIndex < articleIndex ? 1 : -1;
				}
				_state.articleIndex = articleIndex;
				_state.article = _state.section.articles[articleIndex];

				var changes = {};
				changes.article = _state.article;
				changes.articleContent = null;
				if(sectionChanged){
					changes.section = _state.section;
				}
				reader.setModel(changes);
				
				_loadCurrentArticle();
			}
		}
	};

	// -------------------------------------------------------------------------------
	// Font Size
	// -------------------------------------------------------------------------------
	
	
	reader.incrFontSize = function(){
		if(reader.fontSize<32){
			reader.fontSize++;
			_updateFontSize();
		}
	};

	reader.decrFontSize = function(){
		if(reader.fontSize>10){
			reader.fontSize--;
			_updateFontSize();
		}
	};

	var _updateFontSize = function(){
		//change the body font-size, tempalte developpers should rely on % or em font-sizes
		jQuery("body").css( "font-size" , reader.fontSize );
		if(reader.issueTemplate){
			reader.issueTemplate.handleSizeChanged();	
		}
	};
	
	// -------------------------------------------------------------------------------
	// End
	// -------------------------------------------------------------------------------

	reader.exitReader = function(){
		reader.notify(AF.ReaderNotifications.EXIT , reader.getCurrentArticleId() );
	};
	
	// -------------------------------------------------------------------------------
	// app communication
	// -------------------------------------------------------------------------------
	

	// JS => App
	// http://developer.android.com/guide/webapps/webview.html#UsingJavaScript

	AF.ReaderNotifications = {
		//sent when reader object is created (wait for it before calling reader.execute)
		READY: "ready",
		//called when the template and the document are loaded
		LOADED: "loaded",
		//called when the user clicks on the home button
		EXIT: "exit", // , articleId
        // caled when user select a new article
        CURRENT: "currentArticleId" // , current articleId
	};
	
	reader.notify = function(args){
		if(typeof window.Android !== "undefined" && typeof Android.handleReaderNotification !== "undefined"){
			Android.handleReaderNotification(arguments);
		}
        if(AF.notificatorHandler && typeof AF.notificatorHandler.handleReaderNotification !== "undefined"){
            AF.notificatorHandler.handleReaderNotification(arguments);
        }
	};
	
	
	// App => JS
	// Android: myWebView.loadUrl("javascript:AF.reader.execute('nextArticle')");
	
	AF.ReaderPublicAPI = [
		//call this ONCE in order to load an issue
		"loadIssue", // issueFolder , documentFolder , [articleId]
		
		//issue getters
		"getSectionIndex",
		"getArticleIndex",
		"getCurrentArticleId",
		
		//issue navigation
		"gotoArticle", //articleId
		"nextSection",
		"prevSection",
		"nextArticle",
		"prevArticle",
		
		//font size
		"incrFontSize",
		"decrFontSize"
	];
	
	reader.execute = function(functionName , args){
		//calls any function defined in AF.ReaderPublicAPI
		if(AF.ReaderPublicAPI.indexOf(functionName) >= 0){
			var callArgs = Array.prototype.slice.call(arguments , 1);
			return reader[functionName].apply( reader , callArgs );
		}
	};
	
	/*
	//Test / console
	AF.reader.execute("loadIssue","testDocuments/ouestFranceOld/issue.json" , "testTemplates/ouestFranceTemplate" , "redac140.json");
	AF.reader.execute("getSectionIndex");
	AF.reader.selectArticle(1,1);
	 */

	//save a reference to the reader as a global ref
	AF.reader = reader;
	
	AF.reader.notify(AF.ReaderNotifications.READY);
	
	return reader;
});
