define( function(){


	// -------------------------------------------------------------------------------
	// Base Class
	// -------------------------------------------------------------------------------
	
	var BaseSlideshow = function(){
		
		this.padding = 0;
		this.$element = jQuery('<div class="base-slideshow"></div>');
		
		this._buildInnerDOM();
		
		
		//Set up mouse events
		var self = this;
		
		//We have to handle both click and swipe.
		//We have to make sure swipe isn't triggered in top view (changing article instead of slideshow index)
		self.__mousestartX = 0;
		self.__clicked = false;
		var mouseUp = function(event){

			self.$element.off(AF.mousemove,mouseMove);
			self.$element.off(AF.mouseup,mouseUp);

			if(!self.__clicked){ //IE seems to click twice. Sometimes.
				self.onClick();

				event.stopPropagation();

				self.__clicked = true;
			}
		};
		var mouseMove = function(event){

			self.$element.off(AF.mousemove,mouseMove);
			self.$element.off(AF.mouseup,mouseUp);

			event.stopPropagation();

			var eventPos = AF.getEventPosObject(event);

			if(eventPos.clientX == self.__mousestartX){
				//Dumb: meaning only IE goes through here, other browser go straight to mouseup
				self.onClick();
			}else{
				if(eventPos.clientX < self.__mousestartX){
					self.next();
				}else{
					self.prev();
				}
			}
		};
		this.$element.on(AF.mousedown , function(event){

			self.__clicked = false;

			self.$element.one(AF.mousemove,mouseMove);
			self.$element.one(AF.mouseup,mouseUp);

			event.stopPropagation();

			var eventPos = AF.getEventPosObject(event);
			self.__mousestartX = eventPos.clientX;
		});
		
	};
	
	BaseSlideshow.prototype._buildInnerDOM = function(){
		
		this.$imageLayer= jQuery("<div class='image-layer'></div>")
			.appendTo(this.$element);
		
		this.$slideShowNavigation = jQuery("<div class='navigation-layer'></div>")
			.appendTo(this.$element)
			.on(AF.mousedown,function(event){
				//when user clicks in the navigation bar, we don't want event to propagate (and trigger fullscreen)
				event.stopPropagation();
			});
	};

	BaseSlideshow.prototype.onClick = function(){
		//for override	
	};

	BaseSlideshow.prototype.init = function(imageUrls){
		this.imageUrls = imageUrls;
		
		//cleanup
		this.$imageLayer.empty();
		this.$slideShowNavigation.empty();
		this.$navigationButtons = null;
		this.$images = [];
		
		if(this.imageUrls){
			
			for(var i = 0 ; i<this.imageUrls.length ; i++){
				this.$images.push( this._createImage(i , this.imageUrls[i]) );
			}
			
			if(this.imageUrls.length==1){
				//mono image
				this.$slideShowNavigation.css("display","none");
			}else{
				//slideshow
				this.$slideShowNavigation.css("display","block");
				
				var self = this;
				jQuery.each(this.imageUrls , function(index){
					var $button = jQuery("<span class='navigation-button'></span>")
						.appendTo(self.$slideShowNavigation)
						.click( function(event){
							self.setCurrentIndex(index);
						});
				});
				this.$navigationButtons = jQuery(".navigation-button" , this.$slideShowNavigation);
			}
		}
		this.setCurrentIndex( 0 , true);
	};

	BaseSlideshow.prototype.destroy = function(){
		this.$element.remove();
	};

	BaseSlideshow.prototype.setCurrentIndex = function(index , instant){
		this._currentIndex = index;
		
		if(this.$navigationButtons){
			this.$navigationButtons.removeClass("selected");
			this.$navigationButtons.eq(index).addClass("selected");
		}
		
		var width = this.$element.innerWidth();
		if(instant){
			this.$imageLayer.css("left" , (-index*width)+"px");	
		}else{
			this.$imageLayer.animate({ left : (-index*width)+"px" });
		}	
	};

	BaseSlideshow.prototype.next = function(){
		if(this.imageUrls && this._currentIndex<this.imageUrls.length-1){
			this.setCurrentIndex(this._currentIndex+1);
		}
	};

	BaseSlideshow.prototype.prev = function(){
		if(this.imageUrls && this._currentIndex>0){
			this.setCurrentIndex(this._currentIndex-1);
		}
	};

	BaseSlideshow.prototype.getCurrentIndex = function(){
		return this._currentIndex;	
	};

	BaseSlideshow.prototype._createImage = function(index , url){
		var $image = $(new Image()),
			self = this;
		$image.css({
			position: "absolute",
			left: "-10000px", //hide by default
			top: "0px"
		});
		$image.on("load" , function(){
			$image._originalWidth = $image[0].width;
			$image._originalHeight = $image[0].height;
			$image._loaded = true;
			$image._index = index;

			self._layoutImage($image);

			//setTimeout(function(){
			//	self._layoutImage($image);
			//} , 3000);
		});

		//start loading
		$image[0].src = url;
		//and display
		this.$imageLayer.append($image);

		return $image;
	};
	
	BaseSlideshow.prototype.update = function($parentElement){
		if(this.$element.html() == ""){
			//pb with IE our content has been destroyed...
			this._buildInnerDOM();
			this.init(this.imageUrls);
		}

		
		if(this.$element.parent() != $parentElement){
			$parentElement.append(this.$element);
		}
		this.$parentElement = $parentElement;

		if(this.$images){
			for(var i=0 ; i<this.$images.length ; i++){
				this._layoutImage(this.$images[i]);
			}
		}
		
		//After a resize make sure content is correctly placed
		this.setCurrentIndex(this._currentIndex , true);
	};

	BaseSlideshow.prototype._layoutImage = function($image){
		if($image._loaded){

			var width = this.$element.innerWidth() - 2*this.padding,
				height = this.$element.innerHeight() - 2*this.padding,
				scale = Math.min( width / $image._originalWidth , height / $image._originalHeight);
			
			/*
			//OF, they want upscale
			if(scale>1){
				scale = 1;
			}
			*/
			
			var imgWidth = Math.round( $image._originalWidth*scale ),
				imgHeight = Math.round( $image._originalHeight*scale),
				left = this.padding + $image._index*width + Math.round((width-imgWidth)/2),
				right = this.padding + Math.round((height-imgHeight)/2);
			
			var style = {
				width: imgWidth+"px",
				height: imgHeight+"px",
				left: left+"px",
				top: right + "px"
			};
			
			$image.css(style);
		}
	};

	
	// -------------------------------------------------------------------------------
	// Class actually exposed and displayed
	// -------------------------------------------------------------------------------
	
	var ImageSlideshow = function(){
		this.slideshow = new BaseSlideshow();

		this.fullScreen = false;
		this.fullScreenSlideshow = null;
		this.$fullScreenLayer = null;
		this.fullScreenDirty = false;
		
		var self = this;
		this.slideshow.onClick = function(){ self.onClick() };
	};
	
	ImageSlideshow.prototype.init = function(imageUrls){
		this.slideshow.init(imageUrls);
		
		if(this.$fullScreenLayer){
			this.$fullScreenLayer.detach();
		}
		this.fullScreenDirty = true;
	};

	ImageSlideshow.prototype.destroy = function(){
		this.slideshow.destroy();
		this.slideshow = null;
		
		if(this.$fullScreenLayer){
			this.$fullScreenLayer.remove();
			this.$fullScreenLayer = null;
		}
	};
	
	ImageSlideshow.prototype.update = function($parentElement){
		//called on resize and after each reflow
		this.slideshow.update($parentElement);
		if(this.fullScreen){
			this.fullScreenSlideshow.update(this.$fullScreenLayer);
		}
	};

	//this.fullScreenSlideshow = new BaseSlideshow();
	ImageSlideshow.prototype.onClick = function(){
		if(!this.fullScreen){
			this.showFullScreen();
		}else{
			this.endFullScreen();
		}
	};

	ImageSlideshow.prototype.onFullScreenClick = function(){
		this.endFullScreen();
	};

	ImageSlideshow.prototype.showFullScreen = function(){
		if(!this.fullScreen){
			this.fullScreen = true;

			//create the fullscreen layer
			if(!this.$fullScreenLayer){
				this.$fullScreenLayer = jQuery("<div class='slideshow-fullscreen'></div>")
					.appendTo("body"); 

				this.fullScreenSlideshow = new BaseSlideshow();
				this.fullScreenSlideshow.padding = 10;
				
				var self = this;
				this.fullScreenSlideshow.onClick = function(){ self.endFullScreen() };
			}
			if(this.fullScreenDirty){
				this.fullScreenDirty = false;
				this.fullScreenSlideshow.init(this.slideshow.imageUrls);
			}
			this.$fullScreenLayer.appendTo("body");
			
			this.$fullScreenLayer.hide();
			this.$fullScreenLayer.fadeIn( 500 , function(){});

			this.fullScreenSlideshow.update(this.$fullScreenLayer);
			this.fullScreenSlideshow.setCurrentIndex(this.slideshow.getCurrentIndex() , true);
		}
	};
	
	ImageSlideshow.prototype.endFullScreen = function(){
		if(this.fullScreen){
			this.fullScreen = false;
			this.slideshow.setCurrentIndex(this.fullScreenSlideshow.getCurrentIndex() , true);
			
			var self = this;
			this.$fullScreenLayer.fadeOut( 500 , function(){
				self.$fullScreenLayer.detach();
			});
		}
	};
	
	

	AF.widgets.imageSlideshow = ImageSlideshow;
});
